#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include "gtkftree.h"
#include "fileicon.h"
#include <stdlib.h>
#include <gdk/gdkkeysyms.h>

#define SIZE 256

enum {
  OPENED=2,
  HIDE_DIR,
  HAS_DIR,
  OPENED_HIDE_DIR,
  OPENED_HAS_DIR
};

static void build(GtkFtree *,GtkCTreeNode *,gchar *);
static void expand_tree(GtkCTree*,GtkCTreeNode*);
static void gtk_ftree_class_init(GtkFtreeClass *klass);
static void gtk_ftree_init(GtkFtree *ftree);
static void des(GtkObject *widget);
static gint slash_num(gchar *path);
static gchar* node_path(GtkCTree *ctree,GtkCTreeNode *node);
static void k_press(GtkWidget *,GdkEventKey *);

static GtkCTreeClass *parent_class = NULL;


GtkType gtk_ftree_get_type(void)
{
  static GtkType ftree_type=0;

  if (!ftree_type) {
    GtkTypeInfo ftree_info =
    {
      "GtkFtree",
      sizeof (GtkFtree),
      sizeof (GtkFtreeClass),
      (GtkClassInitFunc)  gtk_ftree_class_init,
      (GtkObjectInitFunc) gtk_ftree_init,
      /* reserved_1 */    NULL,
      /* reserved_2 */    NULL,
      (GtkClassInitFunc)  NULL,
    };
    ftree_type = gtk_type_unique (gtk_ctree_get_type (), &ftree_info);
  }
  return ftree_type;
}

static void gtk_ftree_class_init (GtkFtreeClass	*class) 
{
  GtkObjectClass *object_class;
  GtkWidgetClass	*widget_class;

  object_class=(GtkObjectClass*)class;
  widget_class = (GtkWidgetClass*) class;
  parent_class = gtk_type_class (gtk_ctree_get_type ());

  object_class->destroy=des;
}

GtkWidget* gtk_ftree_new (void) 
{
  GtkFtree	*ftree;
  ftree = gtk_type_new(gtk_ftree_get_type ());

  return GTK_WIDGET(ftree);

}


static void gtk_ftree_init(GtkFtree *ftree)
{
  char buff[SIZE];
  char *ff;
  GdkColormap *colormap;
  GtkCTreeNode *parent;
  char *text[1]={"/"};
  GtkWidget *eee,*widget;

  getcwd(buff,SIZE);
  ftree->current_path=(gchar*)g_malloc(strlen(buff)+1);
  strcpy(ftree->current_path,buff);
  
  ftree->flag=FALSE;

  colormap = gdk_colormap_get_system ();

  gtk_ctree_construct (GTK_CTREE (ftree), 1, 0, NULL);
  
  ftree->open_folder= 
    gdk_pixmap_colormap_create_from_xpm_d(NULL,colormap,
					  &ftree->open_folder_mask,
					  NULL,book_open_xpm);
  ftree->close_folder= 
    gdk_pixmap_colormap_create_from_xpm_d(NULL,colormap,
					  &ftree->close_folder_mask,
					  NULL,book_closed_xpm);

  gtk_clist_set_column_auto_resize (GTK_CLIST (ftree), 0, TRUE);
  gtk_clist_set_selection_mode (GTK_CLIST (ftree), GTK_SELECTION_BROWSE);
  gtk_ctree_set_line_style (GTK_CTREE (ftree), GTK_CTREE_LINES_DOTTED);
  
  gtk_signal_connect (GTK_OBJECT (ftree),"tree_expand", 
		      GTK_SIGNAL_FUNC (expand_tree), NULL);
  gtk_signal_connect(GTK_OBJECT(ftree),"key_press_event",
		     GTK_SIGNAL_FUNC(k_press),NULL);

  parent = gtk_ctree_insert_node (GTK_CTREE (ftree),NULL,NULL,
						 text,5,
				  ftree->close_folder,ftree->close_folder_mask,
				  ftree->open_folder,ftree->open_folder_mask,
				  FALSE,FALSE);
  gtk_ctree_select(GTK_CTREE(ftree),parent);
  gtk_ctree_set_indent(GTK_CTREE(ftree),19);
  gtk_ctree_set_spacing(GTK_CTREE(ftree),8);
   
  build(ftree,parent,text[0]); 

  gtk_ctree_expand(GTK_CTREE(ftree),parent);

}

void build(GtkFtree *ftree,GtkCTreeNode *parent,gchar *path)
{
  GtkCTreeNode *sibling=NULL;
  DIR *dir,*tmp;
  struct dirent *ent;
  struct stat st;
  gchar *text[1];
  gint num,a;
  gchar *cat;
  char qq[100];

  if( ( dir=opendir(path))==NULL) return;
  chdir(path);
  while( (ent=readdir(dir))!=NULL)
    {
      
      if(!strcmp(ent->d_name,".")||!strcmp(ent->d_name,"..")) continue;

      stat(ent->d_name,&st);
	  
      if(S_ISDIR(st.st_mode) && (readlink(ent->d_name,qq,100)==-1 ))
	{
	
	    if(ent->d_name[0]== '.' )
	      { 
		a=TRUE;
		if(ftree->flag==FALSE)	
		  continue;
	      }

	  text[0]=ent->d_name;
	  
	  sibling=
	    gtk_ctree_insert_node(GTK_CTREE(ftree),
				  parent,sibling,
				  text,5,ftree->close_folder,
				  ftree->close_folder_mask,
				  ftree->open_folder,
				  ftree->open_folder_mask,
				  FALSE,FALSE);
	  if(ent->d_name[0]=='.')
	    gtk_ctree_node_set_row_data(GTK_CTREE(ftree),sibling,
					(gpointer)HIDE_DIR);
	}
    }



  if(a==TRUE)
    if(HIDE_DIR!=(int)gtk_ctree_node_get_row_data(GTK_CTREE(ftree),parent))
      gtk_ctree_node_set_row_data(GTK_CTREE(ftree),parent,(gpointer)HAS_DIR);

  closedir(dir);
  chdir(ftree->current_path);
  gtk_ctree_sort_node(GTK_CTREE(ftree),parent);
}

void expand_tree(GtkCTree *ctree,GtkCTreeNode *node)
{
  GtkCTreeNode *child,*parent,*sibling;
  gchar *text;
  GString *gs,*gt;
  GdkPixmap *pix;
  GdkBitmap *bit;
  GtkCTreeRow *row;
  gint a=(int)gtk_ctree_node_get_row_data(ctree,node);

  if( a==OPENED||a==OPENED_HAS_DIR||a==OPENED_HIDE_DIR) return;
  else if(a==HAS_DIR) gtk_ctree_node_set_row_data(ctree,node,
						  (gpointer)OPENED_HAS_DIR);
  else if(a==HIDE_DIR) gtk_ctree_node_set_row_data(ctree,node,
					    (gpointer)OPENED_HIDE_DIR);
  else  gtk_ctree_node_set_row_data(ctree,node,(gpointer)OPENED);

  gtk_clist_freeze (GTK_CLIST(ctree));
  gtk_ctree_node_get_pixtext(ctree,node,0,&text,NULL,&pix,&bit);
  gs=g_string_new(text);

  row=GTK_CTREE_ROW(node);
  while((parent=row->parent)!=NULL)
    {
      gtk_ctree_node_get_pixtext(ctree,parent,0,&text,NULL,&pix,&bit);
      gs=g_string_prepend(gs,"/") , gs=g_string_prepend(gs,text);
      row=GTK_CTREE_ROW(parent);
    }

 
  row=GTK_CTREE_ROW(node);
  child=row->children;
  gtk_ctree_node_get_pixtext(ctree,child,0,&text,NULL,&pix,&bit);

  g_string_append(gs,"/");  
  gt=g_string_new(gs->str);
  g_string_append(gt,text);

  build(GTK_FTREE(ctree),child,gt->str);

  g_string_free(gt,TRUE);
  sibling=child;
  while((sibling=GTK_CTREE_ROW(sibling)->sibling)!=NULL)
    {
     
      gtk_ctree_node_get_pixtext(ctree,sibling,0,&text,NULL,&pix,&bit);
      gt=g_string_new(gs->str);
      g_string_append(gt,text);
      
      build( GTK_FTREE(ctree), sibling ,gt->str);
      
      g_string_free(gt,TRUE);
    }
  g_string_free(gs,TRUE);
  gtk_clist_thaw(GTK_CLIST(ctree));
}

void des(GtkObject *widget)
{
  GtkFtree *ftree;
  GtkCTreeNode *node;

  g_return_if_fail(widget!=NULL);

  ftree=GTK_FTREE(widget);
  if(ftree->open_folder)  
    gdk_pixmap_unref(ftree->open_folder);
  if(ftree->close_folder)
    gdk_pixmap_unref(ftree->close_folder);
  if(ftree->open_folder_mask)
    gdk_bitmap_unref(ftree->open_folder_mask);
  if(ftree->close_folder_mask)
    gdk_bitmap_unref(ftree->close_folder_mask);
  if(ftree->current_path)
    {
      chdir(ftree->current_path);  
      free(ftree->current_path);
    }
  GTK_OBJECT_CLASS(parent_class)->destroy(widget);
  
}

gchar* gtk_ftree_get_selected_path(GtkFtree* ftree)
{
  GtkCTree *ctree;
  GtkCList *clist;
  GtkCTreeNode *node;
  gchar *text;
 
  ctree=GTK_CTREE(ftree);
  clist = GTK_CLIST (ctree);

  if(clist->selection)      node=clist->selection->data;
  else                      return NULL;

  gtk_clist_freeze (clist);
  text=node_path(ctree,node);
  gtk_clist_thaw (clist);
  return text;
}

static gchar* node_path(GtkCTree *ctree,GtkCTreeNode *node)
{
  gchar *text;
  GString *gs;
  GtkCTreeRow *row;
  GdkPixmap *pix;
  GdkBitmap *bit;

  gtk_ctree_node_get_pixtext(ctree,node,0,&text,NULL,&pix,&bit);
  gs=g_string_new(text);

  if(GTK_CTREE_ROW(node)->parent)
    {
      row=GTK_CTREE_ROW(node);
      while((node=row->parent)!=NULL)
	{
	  gtk_ctree_node_get_pixtext(ctree,node,0,&text,NULL,&pix,&bit);
	  gs=g_string_prepend(gs,"/") , gs=g_string_prepend(gs,text);
	  row=GTK_CTREE_ROW(node);
	}
      
      text=g_strdup(&gs->str[1]);
      g_string_free(gs,TRUE);
    }
  else 
    {
      text=g_strdup(gs->str);
      g_string_free(gs,TRUE);
    }
  return text;
}



gint gtk_ftree_open(GtkFtree* ftree,gchar *gs)
{
  GtkCTree *ctree;
  GtkCList *clist;
  GtkCTreeNode *node,*sibling,*parent,*nothing;
  GtkCTreeRow *row;
  gchar *path;
  gchar *text;
  GdkPixmap *pix;
  GdkBitmap *bit;
  gshort len;
  gshort num_p=1;
  gshort num_t=1;
  gint flag=-1,noth=FALSE;

  if(path==NULL) return -1;

  ctree=GTK_CTREE(ftree);
  clist=GTK_CLIST(ctree);
  gtk_clist_freeze (clist);
  parent=node=sibling=gtk_ctree_node_nth(ctree,0);
  

  if(strcmp(gs,"/")==0) 	
    {
      gtk_ctree_expand(ctree,node);
      gtk_ctree_select(ctree,node);
      gtk_clist_thaw (clist);
      return 1;
    }

  if(*gs!='/')
    {
      path=(gchar*)g_malloc(strlen(gs)+strlen(ftree->current_path)+2);
      strcpy(path,ftree->current_path),strcat(path,"/"),strcat(path,gs);
    }
  else   
    {
      path=(gchar*)g_malloc(strlen(gs)+1);
      strcpy(path,gs);
    }

  row=GTK_CTREE_ROW(node);

  while(1)
    {
      num_t=slash_num(&path[num_p]);

      if((strncmp(&path[num_p],"..",2)==0 )&&num_t==2)
	{
	  if(GTK_CTREE_ROW(sibling)->expanded==TRUE)
	    gtk_ctree_collapse(ctree,sibling);	  
	  sibling=GTK_CTREE_ROW(sibling)->parent;
	  if(sibling==NULL)sibling=parent;
	}
      else if((strncmp(&path[num_p],".",1)==0)&&(num_t==1));
      else if((strncmp(&path[num_p],"/",1)==0)&&(num_t==0));
      else if (noth==FALSE)
	{
	  sibling=node=row->children;

	  do{
	    gtk_ctree_node_get_pixtext(ctree,sibling,0,&text,NULL,&pix,&bit);
	    len=strlen(text);
	    if(num_t==len)
	      if(strncmp(&path[num_p],text,len)==0) break;
	    sibling=GTK_CTREE_ROW(sibling)->sibling;
	  }while(sibling!=NULL);
	}

      else 
	{
	  flag=-1;
	  break;
	}

      if(sibling==NULL) 
	{
	  sibling=GTK_CTREE_ROW(node)->parent;
	  flag=-1;
	  break; 
	}
      
      num_p+=num_t+1;

      if(!path[num_p]||!path[num_p-1]) 
	{
	  if(GTK_CTREE_ROW(sibling)->children!=NULL)
	    if(GTK_CTREE_ROW(sibling)->expanded==FALSE)
	      gtk_ctree_expand(ctree,sibling);
	  flag=1;
	  break;
	}
      else 
	{
	  if(GTK_CTREE_ROW(sibling)->children==NULL)
	    {
	      nothing=sibling;
	      noth=TRUE;
	    }
	  else
	    {
	      if(GTK_CTREE_ROW(sibling)->expanded==FALSE)
		gtk_ctree_expand(ctree,sibling);
	      noth=FALSE;
	    }
	}
      row=GTK_CTREE_ROW(sibling);      

    }
  gtk_ctree_select(ctree,sibling);
  g_free(path);	  
  gtk_clist_thaw (clist);

  return flag; 
}


static gint slash_num(gchar *path)
{
  gint i=0;
  while((*(path+i)!='/')&&(*(path+i)!='\0')) i++;
  return i;
}

GtkCTreeNode* gtk_ftree_get_selected_node(GtkFtree* ftree)
{
  GtkCTree *ctree;
  GtkCList *clist;
  GtkCTreeNode *node;

  ctree=GTK_CTREE(ftree);
  clist = GTK_CLIST (ctree);

  if(clist->selection)      node=clist->selection->data;
  else                      return NULL;
  return node;
}

void gtk_ftree_all_map(GtkFtree *ftree)
{
  GtkCTree *ctree=GTK_CTREE(ftree);
  GtkCTreeNode *node,*sibling;
  GtkCTreeRow *row;
  GList *list;
  DIR *dir;
  struct dirent *ent;
  struct stat st;
  gchar *path,*text[1],*cudir;
  gint i;
  char qq[100];

  if(ftree->flag==TRUE) return;
  gtk_clist_freeze (GTK_CLIST(ctree));
  
  ftree->flag=TRUE;

  list=gtk_ctree_find_all_by_row_data(ctree,NULL,(gpointer)HAS_DIR);

  for(i=0;i<2;i++)
    {
      while(list!=NULL)
	{
	  node=(GtkCTreeNode*)list->data;
	  path=node_path(ctree,node);

	  if( ( dir=opendir(path))==NULL) continue;
	  chdir(path);

	  while( (ent=readdir(dir))!=NULL)
	    {
	      if(!strcmp(ent->d_name,".")||!strcmp(ent->d_name,"..")) continue;

	      if(ent->d_name[0]== '.' )
		{
		  stat(ent->d_name,&st);
		  if(S_ISDIR(st.st_mode) && ((readlink(ent->d_name,qq,100)==-1) ))
		    {
		      text[0]=ent->d_name;
		      sibling=gtk_ctree_insert_node(ctree,
						    node,NULL,
						    text,5,ftree->close_folder,
						    ftree->close_folder_mask,
						    ftree->open_folder,
						    ftree->open_folder_mask,
						    FALSE,FALSE);
		      gtk_ctree_node_set_row_data(ctree,sibling,
						  (gpointer)HIDE_DIR);
		      cudir=node_path(ctree,sibling);
		      build(ftree,sibling,cudir);
		      chdir(path);
		      g_free(cudir);		      
		    }
		}
	    }
	  gtk_ctree_sort_node(ctree,node);    
	  closedir(dir);
	  list=list->next;
	  g_free(path);
	}
      list=  list=gtk_ctree_find_all_by_row_data(ctree,NULL,
						 (gpointer)OPENED_HAS_DIR);
    }

  chdir(ftree->current_path);
  gtk_clist_thaw (GTK_CLIST(ctree));
}

void gtk_ftree_all_unmap(GtkFtree *ftree)
{
  GtkCTree *ctree=GTK_CTREE(ftree);
  GtkCTreeNode *node,*sibling,*parent,*nothing;
  GtkCTreeRow *row;
  GList *list;
  GdkPixmap *pix;
  GdkBitmap *bit;
  gint i;
  if(ftree->flag==FALSE) return;
  gtk_clist_freeze (GTK_CLIST(ctree));
  
  ftree->flag=FALSE;
  
  list=gtk_ctree_find_all_by_row_data(ctree,NULL,(gpointer)HIDE_DIR);

  for(i=0;i<2;i++)
    {
      list= g_list_last(list);
     while(list!=NULL)
	{
	  node=(GtkCTreeNode*)list->data;
	  gtk_ctree_remove_node(ctree,node);
	  list=list->prev;    
	}
     list=gtk_ctree_find_all_by_row_data(ctree,NULL,(gpointer)OPENED_HIDE_DIR);
    }
  gtk_clist_thaw(GTK_CLIST(ctree));
}

static void k_press(GtkWidget *ftree,GdkEventKey *key)
{
  GtkCTreeNode *node;

  switch(key->keyval){

  case GDK_Return:
    {
      if(node=gtk_ftree_get_selected_node(GTK_FTREE(ftree)))
        {
          if(GTK_CTREE_ROW(node)->children )
	    {
	      if( GTK_CTREE_ROW(node)->expanded)
		gtk_ctree_collapse(GTK_CTREE(ftree),node);
	      else gtk_ctree_expand(GTK_CTREE(ftree),node);
	    }        
	}
      break;
    }
 
  case GDK_Left:
    {
      if(node=gtk_ftree_get_selected_node(GTK_FTREE(ftree)))
	if(node=GTK_CTREE_ROW(node)->parent)
	  {
	    gtk_ctree_select(GTK_CTREE(ftree),node);
	    //  GTK_FTREE(ftree)->cur_node=node;
	  }
      break;
    }
  case GDK_Right:
    {
      if(node=gtk_ftree_get_selected_node(GTK_FTREE(ftree)))
	if(node=GTK_CTREE_ROW(node)->children)
	  {
	    gtk_ctree_select(GTK_CTREE(ftree),node);
	    //	    GTK_FTREE(ftree)->cur_node=node;
	  } 
     break;    
    }
  }
}
