#include "gtkgcanvas.h"
#include <gtk/gtk.h>
#include <stdlib.h>

#define DEFAULT_LEFT_POS  4
#define DEFAULT_TOP_POS   4
#define DEFAULT_SPACING   7


enum {
  ARG_0,
  ARG_ACTIVE,
  ARG_DRAW_INDICATOR
};

static void gtk_gcanvas_class_init     (GtkGcanvasClass   *klass);
static void gtk_gcanvas_init           (GtkGcanvas        *gcanvas);
static GtkWidget* gtk_gcanvas_build (GtkWidget *widget,gint column);
static void gtk_gcanvas_destroy(GtkObject *object);
static void gtk_gcanvas_control_pressed(GtkWidget *widget);
static void gtk_gcanvas_normal_pressed(GtkWidget *);


static GtkTableClass *parent_class=NULL;


GtkType
gtk_gcanvas_get_type (void)
{
  static GtkType gcanvas_type = 0;

  if (!gcanvas_type)
    {
      static const GtkTypeInfo gcanvas_info =
      {
	"GtkGcanvas",
	sizeof (GtkGcanvas),
	sizeof (GtkGcanvasClass),
	(GtkClassInitFunc) gtk_gcanvas_class_init,
	(GtkObjectInitFunc) gtk_gcanvas_init,
        /* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gcanvas_type = gtk_type_unique (GTK_TYPE_TABLE, &gcanvas_info);
      gtk_type_set_chunk_alloc (gcanvas_type, 16);
    }

  return gcanvas_type;
}

static void
gtk_gcanvas_class_init (GtkGcanvasClass *klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;

  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;

  parent_class = gtk_type_class (GTK_TYPE_TABLE);

  object_class->destroy=gtk_gcanvas_destroy;

}

static void
gtk_gcanvas_init (GtkGcanvas *gcanvas)
{
  GTK_WIDGET_SET_FLAGS (gcanvas, GTK_CAN_FOCUS|GTK_CAN_DEFAULT);
  //  GTK_WIDGET_UNSET_FLAGS (gcanvas, GTK_NO_WINDOW);

  gcanvas->column=gcanvas->row=0;
  gcanvas->list_row=gcanvas->list_selected=NULL;

}

GtkWidget* gtk_gcanvas_new (gint column)
{
  GtkWidget *widget;

  widget= GTK_WIDGET (gtk_type_new (gtk_gcanvas_get_type ()));
  widget=  gtk_gcanvas_build(widget,column);
  return widget;
}

static GtkWidget* gtk_gcanvas_build (GtkWidget *widget,gint column)
{
  GtkTable *table=GTK_TABLE(widget);
 
  gtk_table_resize(table,0,column);
  gtk_table_set_homogeneous(table,TRUE);

  GTK_GCANVAS(widget)->column=column;
  GTK_GCANVAS(widget)->row=0;

  GTK_GCANVAS(widget)->list_row=  GTK_GCANVAS(widget)->list_selected=NULL;
  return widget;
}

void gtk_gcanvas_resize(GtkGcanvas *gcanvas,gint column)
{
  gtk_table_resize(GTK_TABLE(gcanvas),0,column);
  gcanvas->column=column;
}


void gtk_gcanvas_append(GtkGcanvas *gcanvas,GdkPixmap **pixmap,
			GdkBitmap **bitmap,gchar **name)
{
  GtkTable *table=GTK_TABLE(gcanvas);
  GtkWidget **gs;
  gint i,flag=0;


  gcanvas->row++;
  gtk_table_resize(table,gcanvas->row,gcanvas->column);
  gs=(GtkWidget**)calloc(gcanvas->column,sizeof(GtkWidget*));
  gcanvas->list_row=g_list_append(gcanvas->list_row,(gpointer)gs);


  for(i=0;i<gcanvas->column;i++)
    {
      if(flag==1)
	{
	  gs[i]=NULL;
	  continue;
	}
      else if(gcanvas->row==0)
	{
	  flag=1;
	  gs[i]=NULL;
	  continue;

	}

      if((!pixmap[i]&&!bitmap[i])&&!name[i])
	{
	  gs[i]=NULL;
	  continue;
	}
	  gs[i]=gtk_gimage_new(pixmap[i],bitmap[i],name[i]);
	  gtk_signal_connect(GTK_OBJECT(gs[i]),"control_pressed",
			     GTK_SIGNAL_FUNC(gtk_gcanvas_control_pressed),NULL);
	  gtk_signal_connect(GTK_OBJECT(gs[i]),"normal_pressed",
			     GTK_SIGNAL_FUNC(gtk_gcanvas_normal_pressed),NULL);

	  gtk_table_attach(table,gs[i],i,i+1,gcanvas->row-1,gcanvas->row,
			   0,0,5,5);
	  gtk_widget_show(gs[i]);
	  while(gtk_events_pending())
	    gtk_main_iteration();
	  
    }

}

void gtk_gcanvas_set_row(GtkGcanvas *gcanvas,gint row,GdkPixmap **pixmap,
			GdkBitmap **bitmap,gchar **name)
{
  GtkWidget **gs;
  gint i;

  g_return_if_fail(row<gcanvas->row);
  
  gs=(GtkWidget**)g_list_nth(gcanvas->list_row,row)->data;

  for(i=0;i<gcanvas->column;i++)
    {

      if(!gs[i])
	{
	  if((!pixmap[i]&&!bitmap[i])&&!name[i]) continue;
	  gs[i]=gtk_gimage_new(pixmap[i],bitmap[i],name[i]);
	}
      else 
	gtk_gimage_set_pixlabel(GTK_GIMAGE(gs[i]),pixmap[i],bitmap[i],name[i]);
    }
}

void gtk_gcanvas_set(GtkGcanvas *gcanvas,gint row,gint column,
		     GdkPixmap *pixmap,GdkBitmap *bitmap,gchar *name)
{
  GtkWidget **gs;

  g_return_if_fail(row<gcanvas->row);
  g_return_if_fail(column<gcanvas->column);

  if((!pixmap&&!bitmap)&&!name) return;

  gs=(GtkWidget**)g_list_nth(gcanvas->list_row,row)->data;
  if(!gs[column])
    gs[column]=gtk_gimage_new(pixmap,bitmap,name);
  else 
    gtk_gimage_set_pixlabel(GTK_GIMAGE(gs[column]),pixmap,bitmap,name);
}

void gtk_gcanvas_remove_node(GtkGcanvas *gcanvas,gint row,gint column)
{
  GtkWidget **gs;
  GList *list;

  g_return_if_fail(row<gcanvas->row);
  g_return_if_fail(column<gcanvas->column);

  gs=(GtkWidget**)g_list_nth(gcanvas->list_row,row)->data;

  if(!gs[column]) return;
  if(list=g_list_find(gcanvas->list_selected,gs[column])) 
    gcanvas->list_selected=g_list_remove_link(gcanvas->list_selected,list);
  gtk_widget_destroy(gs[column]);
  gs[column]=NULL;
}

void gtk_gcanvas_remove_last_row(GtkGcanvas *gcanvas)
{
  GtkWidget **gs;
  gint i;
  GList *list,*node;

  g_return_if_fail(gcanvas->row!=0);

  list= g_list_last(gcanvas->list_row);
  gs=(GtkWidget**)list->data;

  for(i=0;i<gcanvas->column;i++)
    {

      if(gs[i])
	{
	  if(node=g_list_find(gcanvas->list_selected,gs[i])) 
	    gcanvas->list_selected=g_list_remove_link(gcanvas->list_selected,
						      node);

	gtk_widget_destroy(gs[i]);
	}
    }

  g_free(gs);

  gcanvas->list_row=g_list_remove_link(gcanvas->list_row,list);

  gcanvas->row--;
  gtk_table_resize(GTK_TABLE(gcanvas),gcanvas->row,gcanvas->column);
}

void gtk_gcanvas_clear(GtkGcanvas *gcanvas)
{
  GList *list;
  GtkWidget **widget;
  gint i;
  GtkTable *table=GTK_TABLE(gcanvas);

  gtk_container_foreach(GTK_CONTAINER(GTK_TABLE(gcanvas)),
			GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);

  for(list=gcanvas->list_row;list;list=list->next)
    {
      widget=(GtkWidget**)list->data;
      /*    
      for(i=0;i<gcanvas->column;i++)
	{
	  if(widget[i])
	    {
	      gtk_widget_destroy(widget[i]);
	    }
	}
      */
      g_free(widget);
    }

      
  if(gcanvas->list_row)  g_list_free(gcanvas->list_row);
  if(gcanvas->list_selected)  g_list_free(gcanvas->list_selected);

  gtk_table_resize(table,0,0);

  gcanvas->column=0;
  gcanvas->row=0;
  
  gcanvas->list_row=NULL;  
  gcanvas->list_selected=NULL;
}


static void gtk_gcanvas_destroy(GtkObject *object)
{
  GtkGcanvas *gcanvas=GTK_GCANVAS(object);
  GList *list;
  GtkWidget **widget;
  gint i;


  //  if(GTK_IS_CONTAINER(GTK_CONTAINER(gcanvas)))
  gtk_container_foreach(GTK_CONTAINER(GTK_TABLE(gcanvas)),
			GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);

  for(list=gcanvas->list_row;list;list=list->next)
    {
      widget=(GtkWidget**)list->data;
      /*    
      for(i=0;i<gcanvas->column;i++)
	{
	  if(widget[i])
	    {
	      gtk_widget_destroy(widget[i]);
	    }
	}
      */
      g_free(widget);
    }

      
  if(gcanvas->list_row)  g_list_free(gcanvas->list_row);
  if(gcanvas->list_selected)  g_list_free(gcanvas->list_selected);

  if(GTK_OBJECT_CLASS(parent_class)->destroy)
    (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
}
      

static void gtk_gcanvas_control_pressed(GtkWidget *widget)
{
  GtkGcanvas *gcanvas=GTK_GCANVAS(widget->parent);
  GtkGimage *gimage=GTK_GIMAGE(widget);

  if(gimage->state==GIMAGE_SELECTED)
    {
      gcanvas->list_selected=g_list_append(gcanvas->list_selected,widget);
    }
  else 
    {
      gcanvas->list_selected=g_list_remove(gcanvas->list_selected,widget);
    }
}

static void gtk_gcanvas_normal_pressed(GtkWidget *widget)
{
  GtkGcanvas *gcanvas=GTK_GCANVAS(widget->parent);
  GList *list=gcanvas->list_selected;
  GtkWidget *gs;

  while(list)
    {
      gs=(GtkWidget*)list->data;
      if(gs!=widget)
	{
	  GTK_GIMAGE(gs)->state=GIMAGE_NORMAL;
	  gtk_widget_draw(gs,NULL);
	}      
      list=list->next;
    }

  if(gcanvas->list_selected)  
    g_list_free(gcanvas->list_selected);
  
  gcanvas->list_selected=g_list_append(NULL,widget);
}

void gtk_gcanvas_all_select(GtkGcanvas *gcanvas)
{
  GtkGimage *gs;
  GList *list;
  GtkWidget **widget;
  gint i;

  if(gcanvas->list_selected)
    {
      g_list_free(gcanvas->list_selected);
      gcanvas->list_selected=NULL;
    }
  
  for(list=gcanvas->list_row;list;list=list->next)
    {
      widget=(GtkWidget**)list->data;
      for(i=0;i<gcanvas->column;i++)
	{
	  if(!widget[i]) continue;

	  GTK_GIMAGE(widget[i])->state=GIMAGE_SELECTED;
	  gtk_widget_draw(widget[i],NULL);
	  gcanvas->list_selected=g_list_append(gcanvas->list_selected,
					       widget[i]);
	}
    }
}

