#include "gtkgimage.h"
#include <gtk/gtk.h>
#include <stdlib.h>

#define CHILD_SPACING     1
#define DEFAULT_LEFT_POS  4
#define DEFAULT_TOP_POS   4
#define DEFAULT_SPACING   7

enum {
  DOUBLE_CLICKED,
  CONTROL_PRESSED,
  NORMAL_PRESSED,
  LAST_SIGNAL
};

static void gtk_gimage_class_init     (GtkGimageClass   *klass);
static void gtk_gimage_init           (GtkGimage        *gimage);
static GtkWidget* gtk_gimage_build (GtkWidget *gimage,GdkPixmap *pix,
			      GdkBitmap *bit,const gchar *label);

static void gtk_gimage_paint          (GtkWidget        *widget,
				       GdkRectangle     *area);
static gint gtk_gimage_expose         (GtkWidget        *widget,
				       GdkEventExpose   *event);
static void gtk_gimage_draw           (GtkWidget        *widget,
				       GdkRectangle     *area);
static gint gtk_gimage_gimage_double_clicked   (GtkWidget        *widget,
						GdkEventButton   *event);
static void gtk_gimage_realize        (GtkWidget        *widget);
static void gtk_gimage_size_request   (GtkWidget        *widget,
				       GtkRequisition   *requisition);
static void gtk_gimage_size_allocate  (GtkWidget        *widget,
				       GtkAllocation    *allocation);

static void gtk_gimage_destroy(GtkObject *object);

static GtkBinClass *parent_class=NULL;
static guint gimage_signals[LAST_SIGNAL] = { 0 };

GtkType
gtk_gimage_get_type (void)
{
  static GtkType gimage_type = 0;

  if (!gimage_type)
    {
      static const GtkTypeInfo gimage_info =
      {
	"GtkGimage",
	sizeof (GtkGimage),
	sizeof (GtkGimageClass),
	(GtkClassInitFunc) gtk_gimage_class_init,
	(GtkObjectInitFunc) gtk_gimage_init,
        /* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gimage_type = gtk_type_unique (GTK_TYPE_BIN, &gimage_info);
      gtk_type_set_chunk_alloc (gimage_type, 16);
    }

  return gimage_type;
}

static void
gtk_gimage_class_init (GtkGimageClass *klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  GtkContainerClass *container_class;


  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;
  container_class = (GtkContainerClass*) klass;

  parent_class = gtk_type_class (GTK_TYPE_BIN);
  
  gimage_signals[DOUBLE_CLICKED] =
    gtk_signal_new ("double_clicked",
                    GTK_RUN_FIRST ,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GtkGimageClass, double_clicked),
                    gtk_signal_default_marshaller,
		    GTK_TYPE_NONE, 0);

  gimage_signals[NORMAL_PRESSED] =
    gtk_signal_new ("normal_pressed",
                    GTK_RUN_FIRST ,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GtkGimageClass, normal_pressed),
                    gtk_signal_default_marshaller,
		    GTK_TYPE_NONE, 0);

  gimage_signals[CONTROL_PRESSED] =
    gtk_signal_new ("control_pressed",
                    GTK_RUN_FIRST ,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GtkGimageClass, control_pressed),
                    gtk_signal_default_marshaller,
		    GTK_TYPE_NONE, 0);

  gtk_object_class_add_signals (object_class, gimage_signals, LAST_SIGNAL);

  object_class->destroy=gtk_gimage_destroy;

  widget_class->draw = gtk_gimage_draw;
  widget_class->button_press_event = gtk_gimage_gimage_double_clicked;
  widget_class->expose_event = gtk_gimage_expose;
  widget_class->realize = gtk_gimage_realize;
  widget_class->size_request = gtk_gimage_size_request;
  widget_class->size_allocate = gtk_gimage_size_allocate;


  klass->double_clicked = NULL;
  klass->control_pressed = NULL;

}


static void
gtk_gimage_init (GtkGimage *gimage)
{
  GTK_WIDGET_SET_FLAGS (gimage,   GTK_RECEIVES_DEFAULT);
  GTK_WIDGET_UNSET_FLAGS (gimage, GTK_NO_WINDOW |GTK_CAN_FOCUS);

  gimage->pixmap=NULL;
  gimage->label=NULL;
  gimage->state=GIMAGE_NORMAL;

}

static void
gtk_gimage_realize (GtkWidget *widget)
{
  GtkGimage *gimage;
  GdkWindowAttr attributes;
  gint attributes_mask;
  gint border_width;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_GIMAGE (widget));

  gimage = GTK_GIMAGE (widget);
  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

  border_width = GTK_CONTAINER (widget)->border_width;

  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.x = widget->allocation.x + border_width;
  attributes.y = widget->allocation.y + border_width;
  attributes.width = widget->allocation.width - border_width * 2;
  attributes.height = widget->allocation.height - border_width * 2;
  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.visual = gtk_widget_get_visual (widget);
  attributes.colormap = gtk_widget_get_colormap (widget);
  attributes.event_mask = gtk_widget_get_events (widget);
  attributes.event_mask |= (GDK_EXPOSURE_MASK |
			    GDK_BUTTON_PRESS_MASK |
			    GDK_BUTTON_RELEASE_MASK |
			    GDK_ENTER_NOTIFY_MASK |
			    GDK_LEAVE_NOTIFY_MASK);

  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

  widget->window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attributes_mask);
  gdk_window_set_user_data (widget->window, gimage);

  widget->style = gtk_style_attach (widget->style, widget->window);
  gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
}

static void
gtk_gimage_size_request (GtkWidget      *widget,
			 GtkRequisition *requisition)
{
  GtkGimage *gimage;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_GIMAGE (widget));
  g_return_if_fail (requisition != NULL);

  gimage = GTK_GIMAGE (widget);

  requisition->width = (GTK_CONTAINER (widget)->border_width + CHILD_SPACING +
			GTK_WIDGET (widget)->style->klass->xthickness) * 2;
  requisition->height = (GTK_CONTAINER (widget)->border_width + CHILD_SPACING +
			 GTK_WIDGET (widget)->style->klass->ythickness) * 2;

  if (GTK_WIDGET_CAN_DEFAULT (widget))
    {
      requisition->width += (GTK_WIDGET (widget)->style->klass->xthickness * 2 +
			     DEFAULT_SPACING);
      requisition->height += (GTK_WIDGET (widget)->style->klass->ythickness * 2 +
			      DEFAULT_SPACING);
    }

  if (GTK_BIN (gimage)->child && GTK_WIDGET_VISIBLE (GTK_BIN (gimage)->child))
    {
      GtkRequisition child_requisition;

      gtk_widget_size_request (GTK_BIN (gimage)->child, &child_requisition);

      requisition->width += child_requisition.width;
      requisition->height += child_requisition.height;
    }
}

static void
gtk_gimage_size_allocate (GtkWidget     *widget,
			  GtkAllocation *allocation)
{
  GtkGimage *gimage;
  GtkAllocation child_allocation;
  gint border_width;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_GIMAGE (widget));
  g_return_if_fail (allocation != NULL);

  widget->allocation = *allocation;
  border_width = GTK_CONTAINER (widget)->border_width;

  if (GTK_WIDGET_REALIZED (widget))
    gdk_window_move_resize (widget->window,
			    widget->allocation.x + border_width,
			    widget->allocation.y + border_width,
			    widget->allocation.width - border_width * 2,
			    widget->allocation.height - border_width * 2);

  gimage = GTK_GIMAGE (widget);

  if (GTK_BIN (gimage)->child && GTK_WIDGET_VISIBLE (GTK_BIN (gimage)->child))
    {
      child_allocation.x = (CHILD_SPACING + GTK_WIDGET (widget)->style->klass->xthickness);
      child_allocation.y = (CHILD_SPACING + GTK_WIDGET (widget)->style->klass->ythickness);

      child_allocation.width = MAX (1, (gint)widget->allocation.width - child_allocation.x * 2 -
	                         border_width * 2);
      child_allocation.height = MAX (1, (gint)widget->allocation.height - child_allocation.y * 2 -
	                          border_width * 2);

      if (GTK_WIDGET_CAN_DEFAULT (gimage))
	{
	  child_allocation.x += (GTK_WIDGET (widget)->style->klass->xthickness +
				 DEFAULT_LEFT_POS);
	  child_allocation.y += (GTK_WIDGET (widget)->style->klass->ythickness +
				 DEFAULT_TOP_POS);
	  child_allocation.width =  MAX (1, (gint)child_allocation.width -
					 (gint)(GTK_WIDGET (widget)->style->klass->xthickness * 2 + DEFAULT_SPACING));
	  child_allocation.height = MAX (1, (gint)child_allocation.height -
					 (gint)(GTK_WIDGET (widget)->style->klass->xthickness * 2 + DEFAULT_SPACING));
	}

      gtk_widget_size_allocate (GTK_BIN (gimage)->child, &child_allocation);
    }
}

GtkWidget* gtk_gimage_new (GdkPixmap *pix,GdkBitmap *bit,const gchar *label)
{
  GtkWidget *widget;

  widget= GTK_WIDGET (gtk_type_new (gtk_gimage_get_type ()));
widget=  gtk_gimage_build(widget,pix,bit,label);
  return widget;

}


static GtkWidget* gtk_gimage_build (GtkWidget *widget,GdkPixmap *pix,
			      GdkBitmap *bit,const gchar *name)
{
  GtkGimage *gimage=GTK_GIMAGE(widget);

  gimage->table=gtk_table_new(2,1,FALSE);
  gtk_widget_show(gimage->table);
  gtk_container_add (GTK_CONTAINER (widget), gimage->table);

  if(pix!=NULL||bit!=NULL)
    {

      gimage->pixmap=gtk_pixmap_new(pix,bit);
      gtk_table_attach(GTK_TABLE(gimage->table),gimage->pixmap,0,1,0,1,
		       GTK_SHRINK|GTK_EXPAND,
		       GTK_SHRINK|GTK_EXPAND,0,0);
      gtk_widget_show (gimage->pixmap);
      gtk_widget_set_usize(gimage->table,100,-1);
      gtk_widget_set_usize(gimage->pixmap,-1,102);


    }
  else gimage->pixmap=NULL;

  if(name!=NULL)
    gimage->label = gtk_label_new (name);
  else 
    gimage->label=gtk_label_new(" ");

  gtk_table_attach(GTK_TABLE(gimage->table),gimage->label,0,1,1,2,
		   GTK_SHRINK|GTK_EXPAND,
		   GTK_SHRINK|GTK_EXPAND,0,5);
  gtk_widget_show (gimage->label);

  return widget;
}

void gtk_gimage_set_pixlabel(GtkGimage *gimage,GdkPixmap *pix,
			     GdkBitmap *bit,const gchar *label)
{
  if(pix!=NULL || bit!=NULL)
    {
      if(gimage->pixmap!=NULL)
	gtk_pixmap_set(GTK_PIXMAP(gimage->pixmap),pix,bit);
      else
	{
	  gimage->pixmap=gtk_pixmap_new(pix,bit);
	  gtk_table_attach(GTK_TABLE(gimage->table),gimage->pixmap,0,1,0,1,
			   GTK_SHRINK|GTK_EXPAND,
			   GTK_SHRINK|GTK_EXPAND,0,0);
	  gtk_widget_show (gimage->pixmap);
	}
      gtk_widget_set_usize(gimage->table,100,-1);
      gtk_widget_set_usize(gimage->pixmap,-1,102);
    }
  if(label!=NULL)
    gtk_label_set_text(GTK_LABEL(gimage->label),label);

}


void gtk_gimage_double_clicked(GtkGimage *gimage)
{
  g_return_if_fail (gimage != NULL);
  g_return_if_fail (GTK_IS_GIMAGE (gimage));

  gtk_signal_emit (GTK_OBJECT (gimage), gimage_signals[DOUBLE_CLICKED]);

}

void gtk_gimage_control_pressed(GtkGimage *gimage)
{
  g_return_if_fail (gimage != NULL);
  g_return_if_fail (GTK_IS_GIMAGE (gimage));

  gtk_signal_emit (GTK_OBJECT (gimage), gimage_signals[CONTROL_PRESSED]);

}

void gtk_gimage_normal_pressed(GtkGimage *gimage)
{
  g_return_if_fail (gimage != NULL);
  g_return_if_fail (GTK_IS_GIMAGE (gimage));

  gtk_signal_emit (GTK_OBJECT (gimage), gimage_signals[NORMAL_PRESSED]);

}

static void gtk_gimage_draw (GtkWidget    *widget,GdkRectangle *area)
{
  GtkGimage *gimage;
  GdkRectangle child_area;
  GdkRectangle tmp_area;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_GIMAGE (widget));
  g_return_if_fail (area != NULL);

  if (GTK_WIDGET_DRAWABLE (widget))
    {
      gimage = GTK_GIMAGE (widget);

      tmp_area = *area;
      tmp_area.x -= GTK_CONTAINER (gimage)->border_width;
      tmp_area.y -= GTK_CONTAINER (gimage)->border_width;

      gtk_gimage_paint (widget, &tmp_area);

      if (GTK_BIN (gimage)->child && gtk_widget_intersect (GTK_BIN (gimage)->child, &tmp_area, &child_area))
	gtk_widget_draw (GTK_BIN (gimage)->child, &child_area);
    }
}


static gint
gtk_gimage_expose (GtkWidget      *widget,
		   GdkEventExpose *event)
{
  GtkBin *bin;
  GdkEventExpose child_event;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_GIMAGE (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (GTK_WIDGET_DRAWABLE (widget))
    {
      bin = GTK_BIN (widget);

      gtk_gimage_paint (widget, &event->area);

      child_event = *event;
      if (bin->child && GTK_WIDGET_NO_WINDOW (bin->child) &&
	  gtk_widget_intersect (bin->child, &event->area, &child_event.area))
	gtk_widget_event (bin->child, (GdkEvent*) &child_event);
    }

  return FALSE;
}


static void
gtk_gimage_paint (GtkWidget    *widget,
		  GdkRectangle *area)
{
  GtkGimage *gimage=GTK_GIMAGE(widget);
  GtkShadowType shadow_type;
  gint width, height;
  gint x, y;
  GdkGC *gc;

  gc=gdk_gc_new(widget->window);  

  //  gdk_gc_set_clip_rectangle (gc, area);

  if (GTK_WIDGET_DRAWABLE (widget))
    {
	
      x = 0;
      y = 0;
      width = widget->allocation.width - GTK_CONTAINER (widget)->border_width * 2;
      height = widget->allocation.height - GTK_CONTAINER (widget)->border_width * 2;
      
      gdk_window_set_back_pixmap (widget->window, NULL, TRUE);
      gdk_window_clear_area (widget->window, area->x, area->y, area->width, area->height);



      if (GTK_WIDGET_CAN_DEFAULT (widget))
	{
	  x += widget->style->klass->xthickness;
	  y += widget->style->klass->ythickness;
	  width -= 2 * x + DEFAULT_SPACING;
	  height -= 2 * y + DEFAULT_SPACING;
	  x += DEFAULT_LEFT_POS;
	  y += DEFAULT_TOP_POS;
	}


      if(gimage->state==GIMAGE_SELECTED)
	{
	  GdkColor col={0,0,0,0xffff};
	  gdk_color_alloc(gdk_colormap_get_system(),&col);
	  gdk_gc_set_foreground(gc,&col);      	  
    	  gdk_gc_set_line_attributes(gc,2,0,0,0);
	  gdk_draw_line(widget->window,gc,2,
			gimage->label->allocation.y-5,
			width-2,
			gimage->label->allocation.y-5);
	  gdk_draw_rectangle (widget->window, gc, FALSE,
			      x+2, y+3, width-3, height-4);
	}
      else if(gimage->state==GIMAGE_INSENTIVE)
	{

	  gdk_gc_set_line_attributes(gc,2,0,0,0);
	  gdk_draw_line(widget->window,gc,2,
			gimage->label->allocation.y-5,
			width-2,
			gimage->label->allocation.y-5);
	  gdk_gc_set_line_attributes(gc,2,GDK_LINE_ON_OFF_DASH,0,0);
	  gdk_gc_set_dashes(gc,0,"\1\1",2);
	  gdk_draw_rectangle (widget->window, gc, FALSE,
			      x+2, y+3, width-3, height-4);
	}
      else 
	{
	  gdk_gc_set_line_attributes(gc,2,0,0,0);
	  gdk_draw_line(widget->window,gc,2,
			gimage->label->allocation.y-5,
			width-2,
			gimage->label->allocation.y-5);
	  gdk_draw_rectangle (widget->window, gc, FALSE,
			      x+2, y+3, width-3, height-4);
	}
    }

  //  gdk_gc_set_clip_rectangle (gc, NULL);
  gdk_gc_unref(gc);
  
}

static gint
gtk_gimage_gimage_double_clicked (GtkWidget      *widget,
				  GdkEventButton *event)
{
  GtkGimage *gimage=GTK_GIMAGE(widget);
  GtkButton *button;
  gchar *fff;
  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_GIMAGE (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (event->type == GDK_2BUTTON_PRESS)
    {     
      if(event->button==1)
	gtk_gimage_double_clicked (gimage);
	
    }
  else if(event->type == GDK_BUTTON_PRESS)
    {

      if(event->button==1)
	{
	  if(event->state==GDK_CONTROL_MASK)
	    {

	      if(gimage->state!=GIMAGE_SELECTED)
		gimage->state=GIMAGE_SELECTED;
	      else gimage->state=GIMAGE_NORMAL;
	      gtk_gimage_control_pressed(gimage);

	    }
	  else 
	    {
	      gimage->state=GIMAGE_SELECTED;
	      gtk_gimage_normal_pressed(gimage);
	    }
	}
      gtk_widget_draw(widget,NULL);
    }
  return TRUE;
}

static void gtk_gimage_destroy(GtkObject *object)
{
  GtkGimage *gimage=GTK_GIMAGE(object);

  if(gimage->pixmap)
    gtk_widget_destroy(gimage->pixmap);
  if(gimage->label)
    gtk_widget_destroy(gimage->label);
  if(gimage->table)
    gtk_widget_destroy(gimage->table);

  if(GTK_OBJECT_CLASS(parent_class)->destroy)
    (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
}
