#include "gplayer.h"
#include <gdk/gdkkeysyms.h>

static gint next_image();
static GtkWidget* image_create(gchar *path,int full);
static void slide_quit();
static void ne_or_pre();

static GtkWidget *slide_window=NULL;
static GtkWidget *pixmap=NULL;
static gint number=0;

void g_start()
{
  guint time_id;

  if(!image_num)return;
  slide_window=gtk_window_new(GTK_WINDOW_POPUP);
  gtk_widget_set_usize(slide_window,gdk_screen_width(),gdk_screen_height());
  time_id=gtk_timeout_add(gplayer.time*1000,(GtkFunction)(next_image),NULL);

  gtk_widget_set_events(slide_window,GDK_BUTTON_PRESS_MASK|GDK_KEY_PRESS_MASK);
  gtk_signal_connect(GTK_OBJECT(slide_window),"button_press_event",
		     GTK_SIGNAL_FUNC(ne_or_pre),NULL);
  gtk_signal_connect(GTK_OBJECT(slide_window),"key_press_event",
		     GTK_SIGNAL_FUNC(slide_quit),(gpointer)time_id);

  gtk_grab_add(slide_window);
  number=image_num-1;
  next_image();
  gtk_widget_show(slide_window);
  gtk_main();
}

static gint next_image()
{
  GtkWidget *gs;
  gchar *name,*path,*filename;
  GList *list;
  if(pixmap)gtk_widget_destroy(pixmap);
  pixmap=NULL;
  list=g_list_nth(GTK_TABLE(GTK_GCANVAS(gcanvas))->children,number);
  gs=((GtkTableChild*)list->data)->widget;
  path=gtk_ftree_get_selected_path(GTK_FTREE(ftree));
  gtk_label_get(GTK_LABEL(GTK_GIMAGE(gs)->label),&filename);
  name=g_strconcat(path,"/",filename,NULL);
  g_free(path);
  pixmap=image_create(name,gplayer.full);
  g_free(name);
  if(pixmap)
    {
      gtk_container_add(GTK_CONTAINER(slide_window),pixmap);
      gtk_widget_show(pixmap);
    } 
  number--;
  if(number<0) number=image_num-1;
  return TRUE;
}

static GtkWidget* image_create(gchar *path,int full)
{
  GdkImlibImage *image=NULL;
  GtkWidget *pixmap1=NULL;
  GdkPixmap *pix=NULL;
  GdkBitmap *bit=NULL;
  gint w,h;

  image=gdk_imlib_load_image(path);
  if(image==NULL)return NULL;
  
  w=image->rgb_width,h=image->rgb_height;

  if(w>gdk_screen_width()||h>gdk_screen_height())
    {
      double x;
      x=(double)gdk_screen_width()/w;
      if((int)(x*h)<=gdk_screen_height())
	{
	  w=gdk_screen_width();
	  h*=x;
	}
      else
	{
	  x=(double)gdk_screen_height()/h;
	  w*=x;
	  h=gdk_screen_height();
	}
    }

  if(full==FALSE)
    gdk_imlib_render(image,w,h);
  else 
    {
      double wper;
      wper=gdk_screen_width()/(double)w;
      if((int)(wper*h)<=gdk_screen_height())
	{
	  gdk_imlib_render(image,gdk_screen_width(),(int)(wper*h));
	}
      else 
	{
	  double hper;
	  hper=gdk_screen_height()/(double)h;
	  gdk_imlib_render(image,(int)(hper*w),gdk_screen_height());
	}
    }
  pix=gdk_imlib_copy_image(image);
  bit=gdk_imlib_copy_mask(image);
  if(pix)
    pixmap1=gtk_pixmap_new(pix,bit);
  if(pix) gdk_imlib_free_pixmap(pix);
  if(bit) gdk_imlib_free_bitmap(bit);
  gdk_imlib_destroy_image(image);
  return pixmap1;
}

static void ne_or_pre(GtkWidget *widget,GdkEventButton *event)
{
  if(event->button==1)
    next_image();
  else if(event->button==3)
    {
      number+=2;
      if(number>image_num-1)number=number-image_num;
      next_image();
    }
}

static void slide_quit(GtkWidget *a,GdkEventKey *event,gpointer time)
{
  guint time_id=(guint)time;

  if(event->keyval==GDK_n)
    {
      GdkFont *font=gdk_fontset_load("-adobe-*-bold-r-*--130-*-*-*-*-*-*");
      gchar file[50];
      if(number!=image_num-1)
	sprintf(file,"%d / %d",image_num-number-1,image_num);
      else
	sprintf(file,"%d / %d",image_num,image_num);
      gdk_draw_string(slide_window->window,font,
		      slide_window->style->black_gc,gdk_screen_width()/2-250,
		      gdk_screen_height()/2+30,file);
      gdk_font_unref(font);
      return;
    }
  gtk_timeout_remove(time_id);
  number=0;
  gtk_grab_remove(slide_window);

  if(pixmap)gtk_widget_destroy(pixmap);
  if(slide_window)gtk_widget_destroy(slide_window);
  slide_window=NULL;
  pixmap=NULL;
  gtk_main_quit();
}
